'use strict';
const webpack = require('webpack');
const HtmlWebpackPlugin = require('html-webpack-plugin');
const CleanWebpackPlugin = require('clean-webpack-plugin');
const helpers = require('../util/helpers');
const { VueLoaderPlugin } = require('vue-loader');

module.exports = {
  entry: {
    main: [ helpers.root('client/app.js') ],
  },
  resolve: {
    extensions: [ '.js', '.vue', '.json' ],
    alias: {
      vue$: 'vue/dist/vue.esm.js',
    },
  },
  module: {
    rules: [
      {
        test: /\.(js|vue)$/,
        loader: 'eslint-loader',
        enforce: 'pre',
        exclude: /node_modules/,
        options: {
          formatter: require('eslint-friendly-formatter'),
          emitWarning: true,
        },
      },
      {
        test: /\.html$/,
        loader: 'html-loader',
      },
      {
        test: /\.(png|jpe?g|gif|svg|woff|woff2|ttf|eot|ico)$/,
        use: [
          {
            loader: 'url-loader',
            options: {
              limit: 8192,
              name: 'img/[name].[hash:7].[ext]',
            },
          },
        ],
      },
      {
        test: /\.(mp4|webm|ogg|mp3|wav|flac|aac)(\?.*)?$/,
        use: [
          {
            loader: 'url-loader',
            options: {
              limit: 8192,
              name: 'media/[name].[hash:7].[ext]',
            },
          },
        ],
      },
      /* {
        test: /\.(png|jpe?g|gif|svg)$/,
        use: [
          'file-loader',
          'image-webpack-loader',
        ],
      }, */
      {
        test: /\.less$/,
        use: [
          { loader: 'style-loader', options: { singleton: true } },
          'css-loader',
          {
            loader: 'postcss-loader',
            options: {
              plugins: [ require('autoprefixer')({ browsers: 'last 5 version' }) ],
            },
          },
          {
            loader: 'less-loader',
            options: {
              javascriptEnabled: true,
            },
          },
        ],
      },
      {
        test: /\.css$/,
        use: [
          { loader: 'style-loader', options: { singleton: true } },
          'css-loader',
          {
            loader: 'postcss-loader',
            options: {
              plugins: [ require('autoprefixer')({ browsers: 'last 5 version' }) ],
            },
          },
        ],
      },
      {
        test: /\.vue$/,
        loader: 'vue-loader',
      },
      {
        test: /\.js$/,
        exclude: /(node_modules|bower_components)/,
        use: {
          loader: 'babel-loader',
          options: {
            presets: [ '@babel/preset-env' ],
            plugins: [ '@babel/plugin-transform-runtime', '@babel/plugin-syntax-dynamic-import' ],
          },
        },
      },
    ],
  },
  plugins: [
    new CleanWebpackPlugin([ 'webapp' ], {
      root: helpers.root(''),
      verbose: true,
      dry: false,
    }),
    new VueLoaderPlugin(),
    new webpack.ProvidePlugin({
      $: 'jquery',
      jQuery: 'jquery',
      'window.jQuery': 'jquery',
    }),
    new HtmlWebpackPlugin({
      chunksSortMode: 'dependency',
      template: helpers.root('client/index.html'),
    }),
  ],
};
